body {
    font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, Oxygen-Sans, Ubuntu, Cantarell, "Helvetica Neue", sans-serif;
}

a{
    color: #6e829b !important;
}
a:hover{
    text-decoration: none;
}

h1, h2, h3, h4, h5, h6 {
    font-weight: inherit;
    color: #29323d;
}

.navbar{
    background-color: #2c3742;
}

.navbar .navbar-brand{
    color: #FFF !important;
}

.navbar .icon-bar{
    background-color: #FFF;
}

.navbar-top-links > li > a:hover, .navbar-top-links > li > a:focus, .navbar-top-links > .open > a, .navbar-top-links > .open > a:hover, .navbar-top-links > .open > a:focus {
    color: rgb(214, 214, 214) !important;
    background-color: #2c3742;
}

.sidebar ul li:not(:first-child) {
    border-top: 1px solid #e7e7e7;
}

.sidebar .main-menu a i{
    height: 24px;
    font-size: 23px;
}

.form-control, .btn, .input-group-addon, .select2-selection, .alert, .panel {
    border-radius: 0 !important;
}

.table > thead > tr > th, .table > thead > tr > td {
    border-bottom-width: 1px !important;
}

.fc-content {
    background-color: #FFF !important;
}

.dropdown a i, .navbar-top-links a i{
    font-size: 18px;
}

.badge{
    background-color: #6e829b;
}
.badge-primary{
    background-color: #33455a;
}

.timeline{
    border-top: 2px solid #33455a;
}

.timeline__step-label{
	border-color: #33455a;
}

.timeline__step-label:before {
	border-top-color: #33455a;
}

.timeline__step-marker{
	border: 4px solid #33455a;
	color: #33455a;
}

.timeline__step.done .timeline__step-marker{
	background: #33455a;
}

.progress{
	background-color: #33455a;
}

#wrapper {
    background-color: #f8f8f8;
    width: 100%;
}
