# -*- coding: utf-8 -*-

# Generated by Django 1.11 on 2017-09-10 15:43


from django.db import migrations
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0012_auto_20170619_1648'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='settings',
            name='sidebar_fixed',
        ),
        migrations.RemoveField(
            model_name='settings',
            name='sidebar_hidden',
        ),
        migrations.RemoveField(
            model_name='settings',
            name='sidebar_mini',
        ),
        migrations.RemoveField(
            model_name='settings',
            name='theme',
        ),
        migrations.AlterField(
            model_name='organization',
            name='ascii',
            field=djangoplus.db.models.fields.SearchField(blank=True, default=b'', editable=False, null=True),
        ),
        migrations.AlterField(
            model_name='settings',
            name='version',
            field=djangoplus.db.models.fields.CharField(max_length=255, null=True, verbose_name='Vers\xe3o do Sistema'),
        ),
        migrations.AlterField(
            model_name='unit',
            name='ascii',
            field=djangoplus.db.models.fields.SearchField(blank=True, default=b'', editable=False, null=True),
        ),
        migrations.AlterField(
            model_name='user',
            name='permission_mapping',
            field=djangoplus.db.models.fields.TextField(default=b'{}', verbose_name='Mapeamento de Permiss\xe3o'),
        ),
    ]
