<script type="text/javascript" src="/static/js/jquery.qrcode.min.js"></script>
<div id="{{ self.text|slugify }}"></div>
<script>jQuery('#{{ self.text|slugify }}').qrcode({width: 100, height: 100, text: "{{ self.text }}"});</script>