# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-03-09 16:13


from django.db import migrations
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0014_auto_20180106_1921'),
    ]

    operations = [
        migrations.AddField(
            model_name='settings',
            name='company',
            field=djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='Empresa'),
        ),
    ]
