{% if objects %}
<table class="table table-striped table-bordered table-hover">
    <thead>
        <tr>
            <th style="text-align: center"><input type="checkbox" title="Selecionar todos" onchange="var is = this.parentNode.parentNode.parentNode.parentNode.parentNode.getElementsByTagName('input');for(var i=0; i<is.length; i++) is[i].checked = this.checked"/><span class="custom-checkbox"></span></th>
			<th>Opções</th>
        </tr>
    </thead>
    <tbody>
    {% for obj in objects %}
		<tr>
			<td align="center">{{ obj.widget }}</td>
			<td align="left">{{ obj|format }}</td>
		</tr>
	{% endfor %}
	</tbody>
</table>
{% endif %}
