# -*- coding: utf-8 -*-

# Generated by Django 1.11 on 2017-06-18 11:38


from django.db import migrations
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0010_auto_20170428_1545'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='permission_mapping',
            field=djangoplus.db.models.fields.TextField(default=b'{}', verbose_name='Mapeamento de Permiss\xe3o'),
        ),
    ]
